@@ -67,6 +67,9 @@ struct tree_info {
   std::vector< int > naRightCount;
   // Contains the count of NA's which fell to the right for each split value
   // (-1 indicates leaf node, 0 indicates no NA's fell that way)
+  std::vector< int > naDefaultDirection;
+  // Contains the default direction for all NA values per split node if
+  // naDirection == TRUE, -1 indicates left and 1 indicates right
   unsigned int seed;
   // The seed that the tree was given (this uniquely identifies each tree
   // so that we can tell them apart. Very important for prediction when
